@@ -1,7 +1,7 @@
 import math
 from typing import Union, List, Dict, Optional
 import time
-import gym
+import gymnasium as gym
 import os
 import numpy as np
 from bayes_opt import BayesianOptimization
