@@ -1,8 +1,9 @@
 from typing import Union, List, Optional
 import time
-import gym
+import gymnasium as gym
 import os
 import numpy as np
+import math
 from stable_baselines3 import PPO
 from stable_baselines3.common.vec_env.vec_monitor import VecMonitor
 from stable_baselines3.common.vec_env.dummy_vec_env import DummyVecEnv
@@ -31,6 +32,7 @@ class PPOAgent(BaseAgent):
     """
     A PPO agent using the implementation from OpenAI baselines
     """
+
     def __init__(self, simulation_env_config: SimulationEnvConfig,
                  emulation_env_config: Union[None, EmulationEnvConfig], experiment_config: ExperimentConfig,
                  training_job: Optional[TrainingJobConfig] = None):
@@ -46,6 +48,12 @@ class PPOAgent(BaseAgent):
         # Setup experiment metrics
         exp_result = ExperimentResult()
         exp_result.plot_metrics.append(agents_constants.COMMON.AVERAGE_RETURN)
+        exp_result.plot_metrics.append(agents_constants.COMMON.RUNNING_AVERAGE_RETURN)
+        exp_result.plot_metrics.append(agents_constants.COMMON.RUNNING_AVERAGE_TIME_HORIZON)
+        exp_result.plot_metrics.append(agents_constants.COMMON.AVERAGE_TIME_HORIZON)
+        exp_result.plot_metrics.append(agents_constants.COMMON.AVERAGE_UPPER_BOUND_RETURN)
+        exp_result.plot_metrics.append(agents_constants.COMMON.AVERAGE_RANDOM_RETURN)
+        exp_result.plot_metrics.append(agents_constants.COMMON.RUNTIME)
         descr = f"Training of policies with PPO using " \
                 f"simulation:{self.simulation_env_config.name}"
 
@@ -81,7 +89,6 @@ class PPOAgent(BaseAgent):
         # Setup gym environment
         config = self.simulation_env_config.simulation_env_input_config
         orig_env = gym.make(self.simulation_env_config.gym_env_name, config=config)
-        print(orig_env)
         env = make_vec_env(env_id=self.simulation_env_config.gym_env_name,
                            n_envs=self.experiment_config.hparams[agents_constants.COMMON.NUM_PARALLEL_ENVS].value,
                            env_kwargs={"config": config}, vec_env_cls=DummyVecEnv)
@@ -89,8 +96,15 @@ class PPOAgent(BaseAgent):
 
         # Training runs, one per seed
         for seed in self.experiment_config.random_seeds:
+            self.start = time.time()
             exp_result.all_metrics[seed] = {}
             exp_result.all_metrics[seed][agents_constants.COMMON.AVERAGE_RETURN] = []
+            exp_result.all_metrics[seed][agents_constants.COMMON.RUNNING_AVERAGE_RETURN] = []
+            exp_result.all_metrics[seed][agents_constants.COMMON.RUNNING_AVERAGE_TIME_HORIZON] = []
+            exp_result.all_metrics[seed][agents_constants.COMMON.AVERAGE_TIME_HORIZON] = []
+            exp_result.all_metrics[seed][agents_constants.COMMON.AVERAGE_UPPER_BOUND_RETURN] = []
+            exp_result.all_metrics[seed][agents_constants.COMMON.AVERAGE_RANDOM_RETURN] = []
+            exp_result.all_metrics[seed][agents_constants.COMMON.RUNTIME] = []
             ExperimentUtil.set_seed(seed)
 
             # Callback for logging training metrics
@@ -109,7 +123,8 @@ class PPOAgent(BaseAgent):
                 save_dir=self.experiment_config.output_dir, exp_execution=self.exp_execution,
                 env=orig_env, experiment_config=self.experiment_config,
                 L=self.experiment_config.hparams[agents_constants.COMMON.L].value,
-                gym_env_name=self.simulation_env_config.gym_env_name
+                gym_env_name=self.simulation_env_config.gym_env_name,
+                start=self.start
             )
 
             # Create PPO Agent
@@ -131,7 +146,8 @@ class PPOAgent(BaseAgent):
                 max_grad_norm=self.experiment_config.hparams[agents_constants.PPO.MAX_GRAD_NORM].value,
                 target_kl=self.experiment_config.hparams[agents_constants.PPO.TARGET_KL].value,
             )
-            if self.experiment_config.player_type == PlayerType.ATTACKER:
+            if self.experiment_config.player_type == PlayerType.ATTACKER \
+                    and "stopping" in self.simulation_env_config.gym_env_name:
                 orig_env.set_model(model)
 
             # Train
@@ -208,9 +224,10 @@ class PPOTrainingCallback(BaseCallback):
     """
     Callback for monitoring PPO training
     """
+
     def __init__(self, exp_result: ExperimentResult, seed: int, random_seeds: List[int],
                  training_job: TrainingJobConfig, exp_execution: ExperimentExecution,
-                 max_steps: int, simulation_name: str,
+                 max_steps: int, simulation_name: str, start: float,
                  states: List[State], actions: List[Action], player_type: PlayerType,
                  env: gym.Env, experiment_config: ExperimentConfig, verbose=0,
                  eval_every: int = 100, eval_batch_size: int = 10, save_every: int = 10, save_dir: str = "",
@@ -237,6 +254,7 @@ class PPOTrainingCallback(BaseCallback):
         :param experiment_config: the experiment configuration
         :param L: num stops if a stopping environment
         :param gym_env_name: name of gym env
+        :param start_time: the start time-stamp
         """
         super(PPOTrainingCallback, self).__init__(verbose)
         self.states = states
@@ -258,6 +276,7 @@ class PPOTrainingCallback(BaseCallback):
         self.experiment_config = experiment_config
         self.L = L
         self.gym_env_name = gym_env_name
+        self.start = start
 
     def _on_training_start(self) -> None:
         """
@@ -282,7 +301,8 @@ class PPOTrainingCallback(BaseCallback):
 
         :return: (bool) If the callback returns False, training is aborted early.
         """
-        if self.player_type == PlayerType.ATTACKER:
+        if self.experiment_config.player_type == PlayerType.ATTACKER \
+                and "stopping" in self.simulation_name:
             self.env.set_model(self.model)
         return True
 
@@ -297,7 +317,8 @@ class PPOTrainingCallback(BaseCallback):
         This event is triggered before updating the policy.
         """
         Logger.__call__().get_logger().info(f"Training iteration: {self.iter}, seed:{self.seed}, "
-                                            f"progress: {round(100*round(self.num_timesteps/self.max_steps,2),2)}%")
+                                            f"progress: "
+                                            f"{round(100 * round(self.num_timesteps / self.max_steps, 2), 2)}%")
         ts = time.time()
         save_path = self.save_dir + f"/ppo_model{self.iter}_{ts}.zip"
 
@@ -309,7 +330,7 @@ class PPOTrainingCallback(BaseCallback):
 
         # Eval model
         if self.iter % self.eval_every == 0:
-            if self.player_type == PlayerType.ATTACKER:
+            if self.player_type == PlayerType.ATTACKER and "stopping" in self.simulation_name:
                 self.env.set_model(self.model)
             policy = PPOPolicy(
                 model=self.model, simulation_name=self.simulation_name, save_path=save_path,
@@ -318,6 +339,10 @@ class PPOTrainingCallback(BaseCallback):
             o = self.env.reset()
             max_horizon = self.experiment_config.hparams[agents_constants.COMMON.MAX_ENV_STEPS].value
             avg_rewards = []
+            avg_horizons = []
+            avg_upper_bounds = []
+            avg_random_returns = []
+            info = {}
             for i in range(self.eval_batch_size):
                 o = self.env.reset()
                 done = False
@@ -326,15 +351,56 @@ class PPOTrainingCallback(BaseCallback):
                 while not done and t <= max_horizon:
                     a = policy.action(o=o)
                     o, r, done, info = self.env.step(a)
-                    cumulative_reward += r
+                    cumulative_reward += r * math.pow(
+                        self.experiment_config.hparams[agents_constants.COMMON.GAMMA].value, t)
                     t += 1
                     Logger.__call__().get_logger().debug(f"t:{t}, a1:{a}, r:{r}, info:{info}, done:{done}")
                 avg_rewards.append(cumulative_reward)
-            avg_R = np.mean(avg_rewards)
-            policy.avg_R = avg_R
-            Logger.__call__().get_logger().info(f"[EVAL] Training iteration: {self.iter}, Average R:{avg_R}")
+                if agents_constants.ENV_METRICS.TIME_HORIZON in info:
+                    avg_horizons.append(info[agents_constants.ENV_METRICS.TIME_HORIZON])
+                else:
+                    avg_horizons.append(-1)
+                if agents_constants.ENV_METRICS.AVERAGE_UPPER_BOUND_RETURN in info:
+                    avg_upper_bounds.append(info[agents_constants.ENV_METRICS.AVERAGE_UPPER_BOUND_RETURN])
+                else:
+                    avg_upper_bounds.append(-1)
+                if agents_constants.ENV_METRICS.AVERAGE_RANDOM_RETURN in info:
+                    avg_random_returns.append(info[agents_constants.ENV_METRICS.AVERAGE_RANDOM_RETURN])
+                else:
+                    avg_random_returns.append(-1)
 
+                avg_upper_bounds.append(info[agents_constants.ENV_METRICS.AVERAGE_UPPER_BOUND_RETURN])
+            avg_R = np.mean(avg_rewards)
+            avg_T = np.mean(avg_horizons)
+            avg_random_return = np.mean(avg_random_returns)
+            avg_upper_bound = np.mean(avg_upper_bounds)
+            policy.avg_R = avg_R
+            time_elapsed_minutes = (time.time() - self.start) // 60
             self.exp_result.all_metrics[self.seed][agents_constants.COMMON.AVERAGE_RETURN].append(round(avg_R, 3))
+            self.exp_result.all_metrics[self.seed][agents_constants.COMMON.AVERAGE_TIME_HORIZON].append(round(avg_T, 3))
+            self.exp_result.all_metrics[self.seed][agents_constants.COMMON.AVERAGE_UPPER_BOUND_RETURN].append(
+                round(avg_upper_bound, 3))
+            self.exp_result.all_metrics[self.seed][agents_constants.COMMON.RUNTIME].append(time_elapsed_minutes)
+            self.exp_result.all_metrics[self.seed][agents_constants.COMMON.AVERAGE_RANDOM_RETURN].append(
+                round(avg_random_return, 3))
+            running_avg_J = ExperimentUtil.running_average(
+                self.exp_result.all_metrics[self.seed][agents_constants.COMMON.AVERAGE_RETURN],
+                self.experiment_config.hparams[agents_constants.COMMON.RUNNING_AVERAGE].value)
+            self.exp_result.all_metrics[self.seed][agents_constants.COMMON.RUNNING_AVERAGE_RETURN].append(
+                round(running_avg_J, 3))
+            running_avg_T = ExperimentUtil.running_average(
+                self.exp_result.all_metrics[self.seed][agents_constants.COMMON.AVERAGE_TIME_HORIZON],
+                self.experiment_config.hparams[agents_constants.COMMON.RUNNING_AVERAGE].value)
+            self.exp_result.all_metrics[self.seed][agents_constants.COMMON.RUNNING_AVERAGE_TIME_HORIZON].append(
+                round(running_avg_T, 3))
+            Logger.__call__().get_logger().info(
+                f"[EVAL] Training iteration: {self.iter}, Avg R:{round(avg_R, 3)}, "
+                f"Running_avg_{self.experiment_config.hparams[agents_constants.COMMON.RUNNING_AVERAGE].value}_R: "
+                f"{round(running_avg_J, 3)}, Avg T:{round(avg_T, 3)}, "
+                f"Running_avg_{self.experiment_config.hparams[agents_constants.COMMON.RUNNING_AVERAGE].value}_T: "
+                f"{round(running_avg_T, 3)}, Avg pi*: {round(avg_upper_bound, 3)}, "
+                f"Avg random R:{round(avg_random_return, 3)}, time elapsed (min): {time_elapsed_minutes}")
+
             self.env.reset()
 
             # Update training job
