@@ -34,6 +34,7 @@ class COMMON:
     MAX_ENV_STEPS = "max_env_steps"
     RUNNING_AVERAGE = "running_average"
     L = "L"
+    NUM_NODES = "num_nodes"
     STOPPING_ENVS = [
         "csle-stopping-game-v1",
         "csle-stopping-game-mdp-attacker-v1",
@@ -41,6 +42,9 @@ class COMMON:
     ]
     RUNNING_AVERAGE_INTRUSION_LENGTH = "running_average_intrusion_length"
     RUNNING_AVERAGE_INTRUSION_START = "running_average_intrusion_start"
+    AVERAGE_TIME_HORIZON = "average_time_horizon"
+    AVERAGE_UPPER_BOUND_RETURN = "average_upper_bound_return"
+    AVERAGE_RANDOM_RETURN = "average_random_return"
     RUNNING_AVERAGE_TIME_HORIZON = "running_average_time_horizon"
     NUM_PARALLEL_ENVS = "num_parallel_envs"
     OPTIMIZER = "optimizer"
@@ -48,6 +52,7 @@ class COMMON:
     SGD = "SGD"
     LEARNING_RATE_EXP_DECAY = "learning_rate_exp_decay"
     LEARNING_RATE_DECAY_RATE = "learning_rate_decay_rate"
+    RUNTIME = "runtime"
 
 
 class PPO:
@@ -350,3 +355,18 @@ class DYNASEC:
     NO_INTRUSION_ALERTS_MEAN = "no_intrusion_alerts_mean"
     INTRUSION_ALERTS_MEAN_BASELINE = "intrusion_alerts_mean_baseline"
     NO_INTRUSION_ALERTS_MEAN_BASELINE = "no_intrusion_alerts_mean_baseline"
+
+
+class ENV_METRICS:
+    """
+    String constants representing environment metrics
+    """
+    RETURN = "R"
+    TIME_HORIZON = "T"
+    STATE = "s"
+    DEFENDER_ACTION = "a1"
+    ATTACKER_ACTION = "a2"
+    OBSERVATION = "o"
+    TIME_STEP = "t"
+    AVERAGE_UPPER_BOUND_RETURN = "average_upper_bound_return"
+    AVERAGE_RANDOM_RETURN = "average_random_return"
